#!/usr/bin/env bash

function usage {
    cat <<EOF 

Usage: xivocc-build.sh <release_type>
where <release_type> is one of 
  - dev
        Build & publish a dev docker image (used by jenkins)
  - rc
        Build & publish a RC docker image (used by jenkins)
  - prod
        Tag a final docker image based on a previous RC (used by jenkins)
EOF
    exit -1
}

function usage_target_version {
    echo "TARGET_VERSION file is missing or invalid, create one containing:"
    echo "  DOCKER_REPOSITORY=<REPOSITORY>"
    echo "  TARGET_VERSION=<VERSION>"
    echo "where "
    echo " - <REPOSITORY> is the docker repository (i.e xivoxc/xucmgt)"
    echo " - <VERSION> is of form <MAJOR>.<MINOR>.<PATCH> (i.e. 2017.03.02)"
    exit -1
}

function check_git_tag {
    GIT_VERSION=$(git describe --exact-match --tags 2>/dev/null)
    if [ -z $GIT_VERSION ] || [ ! "${GIT_VERSION}" == "${TARGET_VERSION}" ]; then
        echo "Git version (${GIT_VERSION}) and target version (${TARGET_VERSION}) mismatch."
        echo "Check the content of the TARGET_VERSION file matches the git tag"
        exit -1
    fi
}

function check_git_tag_rc {
    GIT_VERSION=$(git describe --exact-match --tags 2>/dev/null)
    if [ -z $GIT_VERSION ] || [ ! "${GIT_VERSION}" == "${TARGET_VERSION}-rc" ]; then
        echo "Git version (${GIT_VERSION}) and target version (${TARGET_VERSION}-rc) mismatch."
        echo "Check the content of the TARGET_VERSION file matches the git tag"
        exit -1
    fi
}

function ensure_git_tag_inexistent {
    if git rev-parse "$TARGET_VERSION" >/dev/null 2>&1
    then
        echo "$TARGET_VERSION is already tagged, cannot continue."
        echo "Did you forget to increment minor or patch number ?"
        exit -1
    fi
}

function docker_tag_push {
    echo "Tagging ${DOCKER_REPOSITORY}:${DOCKER_TAG}"
    if ! docker tag ${DOCKER_REPOSITORY}:${TARGET_VERSION} ${DOCKER_REPOSITORY}:${DOCKER_TAG} ; then
        echo "Error while tagging docker ${DOCKER_REPOSITORY}:${TARGET_VERSION} to ${DOCKER_REPOSITORY}:${DOCKER_TAG}"
        exit -1
    fi
    if ! docker push ${DOCKER_REPOSITORY}:${DOCKER_TAG} ; then
        echo "Error while pushing ${DOCKER_REPOSITORY}:${DOCKER_TAG}"
        exit -1
    fi
}

function build {
    echo "Building $TARGET_VERSION (semver ${TARGET_VERSION_SEMVER})"
    export TARGET_VERSION TARGET_VERSION_SEMVER DOCKER_REPOSITORY DOCKER_TAG
    if bash ./docker_build.sh "${TARGET_VERSION}" ; then
        docker_tag_push
    else
        echo "docker_build.sh failed, exiting"
        exit -1
    fi
}

if [ -z $1 ]; then
    echo "Required argument missing: <release_type>"
    usage
fi

if [ ! -f ./TARGET_VERSION ]; then
    usage_target_version
fi

source ./TARGET_VERSION

if [ -z $TARGET_VERSION ]; then
    usage_target_version
fi

if [ -z $DOCKER_REPOSITORY ]; then
    usage_target_version
fi

if [ ! -f ./docker_build.sh ]; then
    echo "docker_build.sh not found !"
    exit -1
fi

# Check version
VARRAY=( ${TARGET_VERSION//./ } )
for i in {0..2}
do
    if [ -z ${VARRAY[$i]} ]; then
        usage_target_version
    fi
done
TARGET_VERSION_SEMVER=${VARRAY[0]#0}.${VARRAY[1]#0}.${VARRAY[2]#0}

DOCKER_BRANCH="${VARRAY[0]}.${VARRAY[1]}"

case $1 in
    dev)
        ensure_git_tag_inexistent
        COMMIT=$(git rev-parse HEAD | cut -c1-7)
        DOCKER_TAG="${DOCKER_BRANCH}.latestdev"
        TARGET_VERSION="${TARGET_VERSION}-dev-${COMMIT}"
        TARGET_VERSION_SEMVER="${TARGET_VERSION_SEMVER}-dev-${COMMIT}"
        build
        ;;

    rc)
        DOCKER_TAG="${DOCKER_BRANCH}.latestrc"
        check_git_tag_rc
        build
        ;;

    prod)
        check_git_tag
        DOCKER_TAG="${DOCKER_BRANCH}.latest"
        if docker inspect ${DOCKER_REPOSITORY}:${TARGET_VERSION} >/dev/null 2>&1 ; then
            docker_tag_push
            # Push exact version for production only
            if ! docker push ${DOCKER_REPOSITORY}:${TARGET_VERSION} ; then
                echo "Failed to push ${DOCKER_REPOSITORY}:${TARGET_VERSION} failed !"
                exit -1
            fi
        else
            echo "Docker image for ${TARGET_VERSION} not found"
            exit -1
        fi
        ;;

    *)
        echo "Invalid <release_type>: $1"
        usage
        ;;
esac
